package org.opengroup.osdu.notification.provider.azure.util;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;

import org.opengroup.osdu.azure.util.AuthUtils;
import org.opengroup.osdu.core.common.model.http.DpsHeaders;
import org.slf4j.MDC;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;

import com.nimbusds.jwt.JWTClaimsSet;

/**
 * NOTE: should be removed after rewriting HandshakeFilter and revert logging.mdccontext.enabled=true, Order of calling
 * the filters becomes: OrderedRequestContextFilter --> HandshakeFilter --> MDC filter --> ... --> TransactionLogFilter
 */
@Component
@Order(-103)
public class Slf4jMDCFilter implements Filter {

  @Autowired
  private DpsHeaders dpsHeaders;
  @Autowired
  private AuthUtils authUtils;

  /**
   * Filter logic.
   *
   * @param servletRequest Request object.
   * @param servletResponse Response object.
   * @param filterChain Filter Chain object.
   */
  @Override
  public void doFilter(final ServletRequest servletRequest, final ServletResponse servletResponse,
      final FilterChain filterChain) throws IOException, ServletException {
    MDC.clear();
    MDC.setContextMap(getContextMap());
    filterChain.doFilter(servletRequest, servletResponse);
  }

  /**
   * Method to create context map for mdc.
   *
   * @return Context map.
   */
  private Map<String, String> getContextMap() {
    final Map<String, String> contextMap = new HashMap<>();
    contextMap.put(DpsHeaders.CORRELATION_ID, dpsHeaders.getCorrelationId());
    contextMap.put(DpsHeaders.DATA_PARTITION_ID, dpsHeaders.getPartitionId());

    String userId = getUserId();
    if (userId != null) {
      contextMap.put("user-id", userId);
    }

    return contextMap;
  }

  /**
   * Get user ID from Authorization payload (JWT token).
   *
   * @return the user ID
   */
  private String getUserId() {
    JWTClaimsSet claimsSet = authUtils.getClaimsFromJwtToken(dpsHeaders.getAuthorization());
    return claimsSet == null ? null : claimsSet.getSubject();
  }
}
