/* Licensed Materials - Property of IBM              */		
/* (c) Copyright IBM Corp. 2020. All Rights Reserved.*/	


package org.opengroup.osdu.notification.provider.ibm.util;

import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLEncoder;
import java.util.HashMap;
import java.util.Map;

import javax.net.ssl.HttpsURLConnection;

import org.apache.http.HttpStatus;
import org.opengroup.osdu.core.common.logging.JaxRsDpsLog;
import org.opengroup.osdu.core.common.model.http.AppException;
import org.opengroup.osdu.core.common.util.IServiceAccountJwtClient;
import org.opengroup.osdu.core.ibm.util.KeyCloakProvider;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import com.google.gson.Gson;
import com.google.gson.JsonObject;

import lombok.extern.slf4j.Slf4j;

@Slf4j
@Component
public class ServiceAccountJwtIBMClientImpl implements IServiceAccountJwtClient {
	
	//private AppProperties config;
	/*@Autowired
	JaxRsDpsLog log;*/
	@Value("${keycloak.url}")
	private String url;
	@Value("${keycloak.realm}")
    private String realm;
	@Value("${keycloak.client_id}")
	private String client_id;
	@Value("${keycloak.client_secert}")
	private String client_secret;
	private String grant_type = "password";
	@Value("${keycloak.user}")
	private String user;
	@Value("${keycloak.password}")
	private String pwd;
	
/*	public ServiceAccountJwtIBMClientImpl(AppProperties config) {
        if (config == null) {
            throw new IllegalArgumentException("AppProperties is null when initializing jwt client.");
        } else {
            this.config = config;
        }
    }*/
	
	@Override
	public String getIdToken(String tenantName) {
	/*	String t1=null;
		try {
			t1=KeyCloakProvider.getToken(user, pwd);
			System.out.println("************t1 "+t1);
			return "Bearer "+t1;
		} catch (IOException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}*/
		String token_endpoint = String.format("https://%s/auth/realms/%s/protocol/openid-connect/token", url, realm);
		URL url;
		String token = null;
		HttpsURLConnection con;
		try {
			url = new URL(token_endpoint);
			con = (HttpsURLConnection) url.openConnection();
			con.setRequestMethod("POST");
			con.setRequestProperty("Content-Type", "application/x-www-form-urlencoded");
			Map<String, String> parameters = new HashMap<>();
			parameters.put("grant_type", grant_type);
			parameters.put("client_id", client_id);
			parameters.put("client_secret", client_secret);
			parameters.put("username", user);
			parameters.put("password", pwd);

			con.setDoOutput(true);
			DataOutputStream out = new DataOutputStream(con.getOutputStream());
			out.writeBytes(getParamsString(parameters));
			out.flush();
			out.close();

			BufferedReader in = new BufferedReader(new InputStreamReader(con.getInputStream()));
			String inputLine;
			StringBuffer content = new StringBuffer();
			while ((inputLine = in.readLine()) != null) {
				content.append(inputLine);
			}
			in.close();

			con.disconnect();

			Gson gson = new Gson();
			JsonObject jobj = gson.fromJson(content.toString(), JsonObject.class);
			token = jobj.get("access_token").getAsString();
			token = "Bearer "+token;
		} catch (MalformedURLException e) {
			log.error("MalformedURLException while generating token"+e.getMessage());
			e.printStackTrace();
		} catch (IOException e) {
			log.error("IOException while generating token"+e.getMessage());
			throw new AppException(HttpStatus.SC_INTERNAL_SERVER_ERROR, "Provided invalid details", "Error generating token");
		}
		return token;
		// TODO : Implement jwtCache for storing generated token
	}

	private static String getParamsString(Map<String, String> params)
            throws UnsupportedEncodingException {
        StringBuilder result = new StringBuilder();

        for (Map.Entry<String, String> entry : params.entrySet()) {
            result.append(URLEncoder.encode(entry.getKey(), "UTF-8"));
            result.append("=");
            result.append(URLEncoder.encode(entry.getValue(), "UTF-8"));
            result.append("&");
        }

        String resultString = result.toString();
        return resultString.length() > 0
                ? resultString.substring(0, resultString.length() - 1)
                : resultString;
    }

}


